@@ -49,6 +49,7 @@ use crate::io_util::new_http_channel;
 use crate::io_util::HttpBodyWriter;
 use crate::object::Metadata;
 use crate::ops::BytesRange;
+use crate::ops::OpCreate;
 use crate::ops::OpDelete;
 use crate::ops::OpList;
 use crate::ops::OpRead;
@@ -61,8 +62,9 @@ use crate::BytesWriter;
 use crate::ObjectMode;
 use crate::ObjectStreamer;
 
-pub const X_MS_BLOB_TYPE: &str = "x-ms-blob-type";
+const X_MS_BLOB_TYPE: &str = "x-ms-blob-type";
 
+/// Builder for azblob services
 #[derive(Default, Clone)]
 pub struct Builder {
     root: Option<String>,
@@ -92,6 +94,9 @@ impl Debug for Builder {
 }
 
 impl Builder {
+    /// Set root of this backend.
+    ///
+    /// All operations will happen under this root.
     pub fn root(&mut self, root: &str) -> &mut Self {
         if !root.is_empty() {
             self.root = Some(root.to_string())
@@ -100,11 +105,19 @@ impl Builder {
         self
     }
 
+    /// Set container name of this backend.
     pub fn container(&mut self, container: &str) -> &mut Self {
         self.container = container.to_string();
 
         self
     }
+
+    /// Set endpoint of this backend.
+    ///
+    /// Endpoint must be full uri, e.g.
+    ///
+    /// - Azblob: `https://accountname.blob.core.windows.net`
+    /// - Azurite: `http://127.0.0.1:10000/devstoreaccount1`
     pub fn endpoint(&mut self, endpoint: &str) -> &mut Self {
         if !endpoint.is_empty() {
             self.endpoint = Some(endpoint.to_string());
@@ -112,6 +125,11 @@ impl Builder {
 
         self
     }
+
+    /// Set account_name of this backend.
+    ///
+    /// - If account_name is set, we will take user's input first.
+    /// - If not, we will try to load it from environment.
     pub fn account_name(&mut self, account_name: &str) -> &mut Self {
         if !account_name.is_empty() {
             self.account_name = Some(account_name.to_string());
@@ -119,6 +137,11 @@ impl Builder {
 
         self
     }
+
+    /// Set account_key of this backend.
+    ///
+    /// - If account_key is set, we will take user's input first.
+    /// - If not, we will try to load it from environment.
     pub fn account_key(&mut self, account_key: &str) -> &mut Self {
         if !account_key.is_empty() {
             self.account_key = Some(account_key.to_string());
@@ -127,6 +150,7 @@ impl Builder {
         self
     }
 
+    /// Consume builder to build an azblob backend.
     pub async fn finish(&mut self) -> Result<Arc<dyn Accessor>> {
         info!("backend build started: {:?}", &self);
 
@@ -134,7 +158,11 @@ impl Builder {
             // Use "/" as root if user not specified.
             None => "/".to_string(),
             Some(v) => {
-                let mut v = Backend::normalize_path(v);
+                let mut v = v
+                    .split('/')
+                    .filter(|v| !v.is_empty())
+                    .collect::<Vec<&str>>()
+                    .join("/");
                 if !v.starts_with('/') {
                     v.insert(0, '/');
                 }
@@ -158,9 +186,13 @@ impl Builder {
         debug!("backend use container {}", &container);
 
         let endpoint = match &self.endpoint {
-            Some(endpoint) => endpoint.clone(),
-            None => "blob.core.windows.net".to_string(),
-        };
+            Some(endpoint) => Ok(endpoint.clone()),
+            None => Err(other(BackendError::new(
+                HashMap::from([("endpoint".to_string(), "".to_string())]),
+                anyhow!("endpoint is empty"),
+            ))),
+        }?;
+        debug!("backend use endpoint {}", &container);
 
         let context = HashMap::from([
             ("container".to_string(), container.to_string()),
@@ -190,7 +222,7 @@ impl Builder {
         }))
     }
 }
-
+/// Backend for azblob services.
 #[derive(Debug, Clone)]
 pub struct Backend {
     container: String,
@@ -206,23 +238,10 @@ impl Backend {
         Builder::default()
     }
 
-    pub(crate) fn normalize_path(path: &str) -> String {
-        let has_trailing = path.ends_with('/');
-
-        let mut p = path
-            .split('/')
-            .filter(|v| !v.is_empty())
-            .collect::<Vec<&str>>()
-            .join("/");
-
-        if has_trailing && !p.eq("/") {
-            p.push('/')
-        }
-
-        p
-    }
     pub(crate) fn get_abs_path(&self, path: &str) -> String {
-        let path = Backend::normalize_path(path);
+        if path == "/" {
+            return self.root.trim_start_matches('/').to_string();
+        }
         // root must be normalized like `/abc/`
         format!("{}{}", self.root, path)
             .trim_start_matches('/')
@@ -241,24 +260,53 @@ impl Backend {
         }
     }
 }
+
 #[async_trait]
 impl Accessor for Backend {
     #[trace("read")]
-    async fn read(&self, args: &OpRead) -> Result<BytesReader> {
-        increment_counter!("opendal_azure_read_requests");
+    async fn create(&self, args: &OpCreate) -> Result<()> {
+        increment_counter!("opendal_azblob_create_requests");
+        let p = self.get_abs_path(args.path());
 
-        let p = self.get_abs_path(&args.path);
+        let req = self.put_blob(&p, 0, Body::empty()).await;
+        let resp = self.client.request(req).await.map_err(|e| {
+            error!("object {} put_object: {:?}", args.path(), e);
+            other(ObjectError::new("read", args.path(), e))
+        })?;
+
+        match resp.status() {
+            StatusCode::CREATED | StatusCode::OK => {
+                debug!("object {} create finished", args.path());
+                Ok(())
+            }
+            _ => Err(parse_error_response_without_body(
+                resp,
+                "write",
+                args.path(),
+            )),
+        }
+    }
+
+    #[trace("read")]
+    async fn read(&self, args: &OpRead) -> Result<BytesReader> {
+        increment_counter!("opendal_azblob_read_requests");
+
+        let p = self.get_abs_path(args.path());
         debug!(
             "object {} read start: offset {:?}, size {:?}",
-            &p, args.offset, args.size
+            &p,
+            args.offset(),
+            args.size()
         );
 
-        let resp = self.get_blob(&p, args.offset, args.size).await?;
+        let resp = self.get_blob(&p, args.offset(), args.size()).await?;
         match resp.status() {
             StatusCode::OK | StatusCode::PARTIAL_CONTENT => {
                 debug!(
                     "object {} reader created: offset {:?}, size {:?}",
-                    &p, args.offset, args.size
+                    &p,
+                    args.offset(),
+                    args.size()
                 );
 
                 Ok(Box::new(
@@ -274,20 +322,20 @@ impl Accessor for Backend {
 
     #[trace("write")]
     async fn write(&self, args: &OpWrite) -> Result<BytesWriter> {
-        let p = self.get_abs_path(&args.path);
-        debug!("object {} write start: size {}", &p, args.size);
+        let p = self.get_abs_path(args.path());
+        debug!("object {} write start: size {}", &p, args.size());
 
         let (tx, body) = new_http_channel();
 
-        let req = self.put_blob(&p, args.size, body).await;
+        let req = self.put_blob(&p, args.size(), body).await;
 
         let bs = HttpBodyWriter::new(args, tx, self.client.request(req), |op, resp| {
             match resp.status() {
                 http::StatusCode::CREATED | http::StatusCode::OK => {
-                    debug!("object {} write finished: size {:?}", &op.path, op.size);
+                    debug!("object {} write finished: size {:?}", op.path(), op.size());
                     Ok(())
                 }
-                _ => Err(parse_error_response_without_body(resp, "write", &op.path)),
+                _ => Err(parse_error_response_without_body(resp, "write", op.path())),
             }
         });
 
@@ -298,13 +346,13 @@ impl Accessor for Backend {
     async fn stat(&self, args: &OpStat) -> Result<Metadata> {
         increment_counter!("opendal_azure_stat_requests");
 
-        let p = self.get_abs_path(&args.path);
+        let p = self.get_abs_path(args.path());
         debug!("object {} stat start", &p);
 
         // Stat root always returns a DIR.
         if self.get_rel_path(&p).is_empty() {
             let mut m = Metadata::default();
-            m.set_path(&args.path);
+            m.set_path(args.path());
             m.set_content_length(0);
             m.set_mode(ObjectMode::DIR);
             m.set_complete();
@@ -317,7 +365,7 @@ impl Accessor for Backend {
         match resp.status() {
             http::StatusCode::OK => {
                 let mut m = Metadata::default();
-                m.set_path(&args.path);
+                m.set_path(args.path());
 
                 // Parse content_length
                 if let Some(v) = resp.headers().get(http::header::CONTENT_LENGTH) {
@@ -339,7 +387,7 @@ impl Accessor for Backend {
                     let v = v.to_str().expect("header must not contain non-ascii value");
                     let t =
                         OffsetDateTime::parse(v, &Rfc2822).expect("must contain valid time format");
-                    m.set_last_modified(t.into());
+                    m.set_last_modified(t);
                 }
 
                 if p.ends_with('/') {
@@ -355,7 +403,7 @@ impl Accessor for Backend {
             }
             StatusCode::NOT_FOUND if p.ends_with('/') => {
                 let mut m = Metadata::default();
-                m.set_path(&args.path);
+                m.set_path(args.path());
                 m.set_content_length(0);
                 m.set_mode(ObjectMode::DIR);
                 m.set_complete();
@@ -376,7 +424,7 @@ impl Accessor for Backend {
 
         let resp = self.delete_blob(&p).await?;
         match resp.status() {
-            StatusCode::NO_CONTENT | StatusCode::ACCEPTED => {
+            StatusCode::ACCEPTED | StatusCode::NOT_FOUND => {
                 debug!("object {} delete finished", &p);
                 Ok(())
             }
@@ -388,11 +436,7 @@ impl Accessor for Backend {
     async fn list(&self, args: &OpList) -> Result<ObjectStreamer> {
         increment_counter!("opendal_azblob_list_requests");
 
-        let mut path = self.get_abs_path(&args.path);
-        // Make sure list path is endswith '/'
-        if !path.ends_with('/') && !path.is_empty() {
-            path.push('/')
-        }
+        let path = self.get_abs_path(&args.path);
         debug!("object {} list start", &path);
 
         Ok(Box::new(AzblobObjectStream::new(self.clone(), path)))
@@ -408,7 +452,7 @@ impl Backend {
         size: Option<u64>,
     ) -> Result<hyper::Response<hyper::Body>> {
         let mut req = hyper::Request::get(&format!(
-            "https://{}.{}/{}/{}",
+            "{}.{}/{}/{}",
             self.account_name, self.endpoint, self.container, path
         ));
 
@@ -439,7 +483,7 @@ impl Backend {
         body: Body,
     ) -> hyper::Request<hyper::Body> {
         let mut req = hyper::Request::put(&format!(
-            "https://{}.{}/{}/{}",
+            "{}.{}/{}/{}",
             self.account_name, self.endpoint, self.container, path
         ));
 
@@ -461,7 +505,7 @@ impl Backend {
         path: &str,
     ) -> Result<hyper::Response<hyper::Body>> {
         let req = hyper::Request::head(&format!(
-            "https://{}.{}/{}/{}",
+            "{}.{}/{}/{}",
             self.account_name, self.endpoint, self.container, path
         ));
         let mut req = req
@@ -479,7 +523,7 @@ impl Backend {
     #[trace("delete_blob")]
     pub(crate) async fn delete_blob(&self, path: &str) -> Result<hyper::Response<hyper::Body>> {
         let req = hyper::Request::delete(&format!(
-            "https://{}.{}/{}/{}",
+            "{}.{}/{}/{}",
             self.account_name, self.endpoint, self.container, path
         ));
 
@@ -502,7 +546,7 @@ impl Backend {
         next_marker: &str,
     ) -> Result<hyper::Response<hyper::Body>> {
         let mut uri = format!(
-            "https://{}.{}/{}?restype=container&comp=list&delimiter=/",
+            "{}.{}/{}?restype=container&comp=list&delimiter=/",
             self.account_name, self.endpoint, self.container
         );
         if !path.is_empty() {
