@@ -13,6 +13,7 @@
 // limitations under the License.
 
 use std::collections::HashMap;
+use std::io::ErrorKind;
 use std::io::Result;
 use std::io::SeekFrom;
 use std::path::PathBuf;
@@ -28,6 +29,7 @@ use log::error;
 use log::info;
 use metrics::increment_counter;
 use minitrace::trace;
+use time::OffsetDateTime;
 use tokio::fs;
 
 use super::error::parse_io_error;
@@ -38,6 +40,7 @@ use crate::error::ObjectError;
 use crate::object::Metadata;
 use crate::object::ObjectMode;
 use crate::object::ObjectStreamer;
+use crate::ops::OpCreate;
 use crate::ops::OpDelete;
 use crate::ops::OpList;
 use crate::ops::OpRead;
@@ -91,13 +94,6 @@ impl Builder {
 }
 
 /// Backend is used to serve `Accessor` support for posix alike fs.
-///
-/// # Note
-///
-/// We will use separate dedicated thread pool (powered by `unblocking`)
-/// for better async performance under tokio. All `std::File` will be wrapped
-/// by `Unblock` to gain async support. IO will happen at the separate dedicated
-/// thread pool, so we will not block the tokio runtime.
 #[derive(Debug, Clone)]
 pub struct Backend {
     root: String,
@@ -109,13 +105,9 @@ impl Backend {
     }
 
     pub(crate) fn get_abs_path(&self, path: &str) -> String {
-        // Joining an absolute path replaces the existing path, we need to
-        // normalize it before.
-        let path = path
-            .split('/')
-            .filter(|v| !v.is_empty())
-            .collect::<Vec<&str>>()
-            .join("/");
+        if path == "/" {
+            return self.root.clone();
+        }
 
         PathBuf::from(&self.root)
             .join(path)
@@ -126,14 +118,69 @@ impl Backend {
 
 #[async_trait]
 impl Accessor for Backend {
+    async fn create(&self, args: &OpCreate) -> Result<()> {
+        let path = self.get_abs_path(args.path());
+
+        let parent = PathBuf::from(&path)
+            .parent()
+            .ok_or_else(|| {
+                other(ObjectError::new(
+                    "write",
+                    &path,
+                    anyhow!("malformed path: {:?}", &path),
+                ))
+            })?
+            .to_path_buf();
+
+        fs::create_dir_all(&parent).await.map_err(|e| {
+            let e = parse_io_error(e, "write", &parent.to_string_lossy());
+            error!(
+                "object {} create_dir_all for parent {}: {:?}",
+                &path,
+                &parent.to_string_lossy(),
+                e
+            );
+            e
+        })?;
+
+        if args.mode() == ObjectMode::FILE {
+            fs::OpenOptions::new()
+                .create(true)
+                .write(true)
+                .open(&path)
+                .await
+                .map_err(|e| {
+                    let e = parse_io_error(e, "write", &path);
+                    error!("object {} create: {:?}", &path, e);
+                    e
+                })?;
+
+            return Ok(());
+        }
+
+        if args.mode() == ObjectMode::DIR {
+            fs::create_dir(&path).await.map_err(|e| {
+                let e = parse_io_error(e, "write", &path);
+                error!("object {} create: {:?}", &path, e);
+                e
+            })?;
+
+            return Ok(());
+        }
+
+        unreachable!()
+    }
+
     #[trace("read")]
     async fn read(&self, args: &OpRead) -> Result<BytesReader> {
         increment_counter!("opendal_fs_read_requests");
 
-        let path = self.get_abs_path(&args.path);
+        let path = self.get_abs_path(args.path());
         debug!(
             "object {} read start: offset {:?}, size {:?}",
-            &path, args.offset, args.size
+            &path,
+            args.offset(),
+            args.size()
         );
 
         let f = fs::OpenOptions::new()
@@ -148,7 +195,7 @@ impl Accessor for Backend {
 
         let mut f = Compat::new(f);
 
-        if let Some(offset) = args.offset {
+        if let Some(offset) = args.offset() {
             f.seek(SeekFrom::Start(offset)).await.map_err(|e| {
                 let e = parse_io_error(e, "read", &path);
                 error!("object {} seek: {:?}", &path, e);
@@ -156,14 +203,16 @@ impl Accessor for Backend {
             })?;
         };
 
-        let r: BytesReader = match args.size {
+        let r: BytesReader = match args.size() {
             Some(size) => Box::new(f.take(size)),
             None => Box::new(f),
         };
 
         debug!(
             "object {} reader created: offset {:?}, size {:?}",
-            &path, args.offset, args.size
+            &path,
+            args.offset(),
+            args.size()
         );
         Ok(Box::new(r))
     }
@@ -172,8 +221,8 @@ impl Accessor for Backend {
     async fn write(&self, args: &OpWrite) -> Result<BytesWriter> {
         increment_counter!("opendal_fs_write_requests");
 
-        let path = self.get_abs_path(&args.path);
-        debug!("object {} write start: size {}", &path, args.size);
+        let path = self.get_abs_path(args.path());
+        debug!("object {} write start: size {}", &path, args.size());
 
         // Create dir before write path.
         //
@@ -214,7 +263,7 @@ impl Accessor for Backend {
                 e
             })?;
 
-        debug!("object {} write finished: size {:?}", &path, args.size);
+        debug!("object {} write finished: size {:?}", &path, args.size());
         Ok(Box::new(Compat::new(f)))
     }
 
@@ -222,7 +271,7 @@ impl Accessor for Backend {
     async fn stat(&self, args: &OpStat) -> Result<Metadata> {
         increment_counter!("opendal_fs_stat_requests");
 
-        let path = self.get_abs_path(&args.path);
+        let path = self.get_abs_path(args.path());
         debug!("object {} stat start", &path);
 
         let meta = fs::metadata(&path).await.map_err(|e| {
@@ -232,16 +281,24 @@ impl Accessor for Backend {
         })?;
 
         let mut m = Metadata::default();
-        m.set_path(&args.path);
         if meta.is_dir() {
+            let mut p = args.path().to_string();
+            if !p.ends_with('/') {
+                p.push('/')
+            }
+            m.set_path(&p);
             m.set_mode(ObjectMode::DIR);
-        } else {
-            // TODO: we should handle LINK or other types here.
+        } else if meta.is_file() {
+            m.set_path(args.path());
             m.set_mode(ObjectMode::FILE);
+        } else {
+            m.set_path(args.path());
+            m.set_mode(ObjectMode::Unknown);
         }
         m.set_content_length(meta.len() as u64);
         m.set_last_modified(
             meta.modified()
+                .map(OffsetDateTime::from)
                 .map_err(|e| parse_io_error(e, "stat", &path))?,
         );
         m.set_complete();
@@ -261,7 +318,7 @@ impl Accessor for Backend {
         let meta = fs::metadata(&path).await;
 
         if let Err(err) = meta {
-            return if err.kind() == std::io::ErrorKind::NotFound {
+            return if err.kind() == ErrorKind::NotFound {
                 Ok(())
             } else {
                 let e = parse_io_error(err, "delete", &path);
@@ -292,8 +349,8 @@ impl Accessor for Backend {
         let path = self.get_abs_path(&args.path);
         debug!("object {} list start", &path);
 
-        let f = fs::read_dir(&path).await.map_err(|e| {
-            let e = parse_io_error(e, "read", &path);
+        let f = std::fs::read_dir(&path).map_err(|e| {
+            let e = parse_io_error(e, "list", &path);
             error!("object {} list: {:?}", &path, e);
             e
         })?;
